<?php
/**
*
* @package phpBB3
* @version $Id: prime_subject_check.php,v 1.5.0 2010/11/22 13:26:00 primehalo Exp $
* @copyright (c) 2008-2010 Ken F. Innes IV
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Include only once.
*/
if (!defined('INCLUDES_PRIME_SUBJECT_CHECK_PHP'))
{
	/**
	* Options
	*/
	define('PRIME_SUBJECT_CHECK_SORT_BEST', true);	// Sort list by best match?
	define('PRIME_SUBJECT_CHECK_MATCH_ALL', false);	// Match all search words?

	/**
	* Constants
	*/
	define('INCLUDES_PRIME_SUBJECT_CHECK_PHP', true);
	define('PRIME_SUBJECT_CHECK_NO',	0);
	define('PRIME_SUBJECT_CHECK_YES',	1);
	define('PRIME_SUBJECT_CHECK_EXACT',	2);

	/**
	* Displays an ACP option.
	*/
	function prime_subject_check_acp_option($key = '', $select_id = 0)
	{
		global $config, $user;

		$user->add_lang('mods/prime_subject_check');
		$options_array = array(
			'subject_check_bypass' => array(
				PRIME_SUBJECT_CHECK_YES		=> 'YES',
				PRIME_SUBJECT_CHECK_NO		=> 'NO',
				PRIME_SUBJECT_CHECK_EXACT	=> 'SUBJECT_CHECK_ACP_BYPASS_EXACT'
			)
		);
		if ($key !== '' && is_array($options_array[$key]))
		{
			$select = '';
			foreach ($options_array[$key] as $idx => $title)
			{
				$selected = ((integer)$config[$key] === $idx) ? ' checked="checked"' : '';
				$select .= '<label><input type="radio"' . ($idx == 1 ? ' id="' . $key . '"' : '') . ' name="config[' . $key . ']" value="' . $idx . '"' . $selected . ' class="radio" /> ' . $user->lang[$title] . '</label>';
			}
			return($select);
		}
	}

	/**
	* Prime Subject Check class structure
	*/
	class prime_subject_check
	{
		var $subject	= false;
		var $bypass		= true;
		var $config		= array();

		/**
		* Constructor
		*/
		function prime_subject_check()
		{
			global $config;

			$this->initialize_options();
			$this->config['enable']		= (boolean)$config['subject_check_enable'];
			$this->config['bypass']		= (integer)$config['subject_check_bypass'];
			$this->config['strict']		= (boolean)$config['subject_check_strict'];
			$this->config['ajax']		= (boolean)$config['subject_check_ajax'];
			$this->config['locked']		= (boolean)$config['subject_check_locked'];
			$this->config['pop']		= (boolean)$config['subject_check_pop'   ];
			$this->config['admin']		= (boolean)$config['subject_check_admin' ];
			$this->config['mod']		= (boolean)$config['subject_check_mod'   ];
			$this->config['forums']		= (boolean)$config['subject_check_forums'];
			$this->config['fnames']		= (boolean)$config['subject_check_fnames'];
			$this->config['chars']		= (integer)$config['fulltext_native_min_chars']; //(integer)$config['subject_check_chars' ]; // Skip words smaller than this number of characters
			$this->config['limit']		= (integer)$config['subject_check_limit' ];
			$this->config['active']		= (integer)$config['subject_check_active'];
			$this->config['postcount']	= (integer)$config['subject_check_count' ];
		}

		/**
		* Set defaults for options if they have not been set
		*/
		function initialize_options()
		{
			global $config;

			$defaults = array(
				// config option		=> default value
				'subject_check_enable'	=> true,
				'subject_check_bypass'	=> PRIME_SUBJECT_CHECK_YES,
				'subject_check_strict'	=> false,
				'subject_check_ajax'	=> true,
				'subject_check_locked'	=> false,
				'subject_check_pop'		=> true,
				'subject_check_admin'	=> false,
				'subject_check_mod'		=> false,
				'subject_check_forums'	=> false,
				'subject_check_fnames'	=> false,
				'subject_check_limit'	=> 0,
				'subject_check_active'	=> 0,
				'subject_check_count'	=> 0,
			);
			foreach ($defaults as $option => $default)
			{
				if (!isset($config[$option]))
				{
					set_config($option, $default);
				}
			}
		}

		/**
		* Display the ACP options.
		*/
		function display_acp_options(&$display_vars)
		{
			global $user;

			$user->add_lang('mods/prime_subject_check');
			$this->initialize_options();

			// Find an available legend number
			for ($legend_i = 1; isset($display_vars['vars']['legend' . $legend_i]); $legend_i++)
			{
			}

			// Display admin options
			$options = array(
				'legend' . ($legend_i - 1)	=> 'SUBJECT_CHECK_ACP_TITLE',
				'subject_check_enable'	=> array('lang' => 'SUBJECT_CHECK_ACP_ENABLE'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_bypass'	=> array('lang' => 'SUBJECT_CHECK_ACP_BYPASS'	,	'validate' => 'int',	'type' => 'custom',	'function' => 'prime_subject_check_acp_option', 'params' => array('{KEY}', '{CONFIG_VALUE}'), 'explain' => true),
				'subject_check_strict'	=> array('lang' => 'SUBJECT_CHECK_ACP_STRICT'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_ajax'	=> array('lang' => 'SUBJECT_CHECK_ACP_AJAX'		,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_pop'		=> array('lang' => 'SUBJECT_CHECK_ACP_POP'		,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_admin'	=> array('lang' => 'SUBJECT_CHECK_ACP_ADMIN'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_mod'		=> array('lang' => 'SUBJECT_CHECK_ACP_MOD'		,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_locked'	=> array('lang' => 'SUBJECT_CHECK_ACP_LOCKED'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_forums'	=> array('lang' => 'SUBJECT_CHECK_ACP_FORUMS'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_fnames'	=> array('lang' => 'SUBJECT_CHECK_ACP_FNAMES'	,	'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'subject_check_limit'	=> array('lang' => 'SUBJECT_CHECK_ACP_LIMIT'	,	'int:0',	'type' => 'text:5:4', 'explain' => true),
				'subject_check_active'	=> array('lang' => 'SUBJECT_CHECK_ACP_ACTIVE'	,	'int:0',	'type' => 'text:5:4', 'explain' => true, 'append' => ' ' . $user->lang['DAYS']),
				'subject_check_count'	=> array('lang' => 'SUBJECT_CHECK_ACP_COUNT'	,	'int:0',	'type' => 'text:5:4', 'explain' => true),
				'legend' . ($legend_i)	=> 'ACP_SUBMIT_CHANGES',
			);
			foreach ($options as $key => $val)
			{
				$display_vars['vars'][$key] = $val;
			}
		}

		/**
		* Split the string into words, clean them up, remove words to ignore, and add in alternate spellings.
		*/
		function clean_and_split_words($entry, &$ignore_words_list, &$synonym_list)
		{
			$entry = ' ' . strip_tags(utf8_case_fold_nfc($entry)) . ' ';

			// Remove non-alphanumeric characters
			if (!class_exists('fulltext_native'))
			{
				global $phpbb_root_path, $phpEx;
				include($phpbb_root_path . 'includes/search/fulltext_native.' . $phpEx);
			}
			$fulltext_native = new fulltext_native(($error = null));
			$entry = $fulltext_native->cleanup($entry);

			// Remove words to be ignored
			if (!empty($ignore_words_list))
			{
				foreach ($ignore_words_list as $word)
				{
					$entry = str_replace(' ' . trim($word) . ' ', ' ', $entry);
				}
			}

			// Add synonyms
			if (!empty($synonym_list))
			{
				foreach ($synonym_list as $synonym1 => $synonym2)
				{
					$synonym1 = trim(utf8_case_fold_nfc($synonym1));
					$synonym2 = trim(utf8_case_fold_nfc($synonym2));
					$synonym1_pos = utf8_strpos($entry, ' ' . $synonym1 . ' ');
					$synonym2_pos = utf8_strpos($entry, ' ' . $synonym2 . ' ');
					$entry .= ($synonym1_pos !== false && $synonym2_pos === false) ? $synonym2 . ' ' : '';
					$entry .= ($synonym2_pos !== false && $synonym1_pos === false) ? $synonym1 . ' ' : '';
				}
			}

			// Reduce multiple spaces to a single space and split this trimmed string into words.
			$word_list = explode(' ', trim(preg_replace('#\s+#', ' ', $entry)));

			return $word_list;
		}

		/**
		* Look for and return a list of topics with similar subjects to the one that is trying to be posted.
		*/
		function look_for_posts()
		{
			global $phpbb_root_path, $phpEx, $db, $user, $auth;
			global $mode, $error, $forum_id, $post_data; // global to posting.php

			$subject = $this->subject = $post_data['post_subject']; // Already utf8 normalized
			$checked = utf8_normalize_nfc(request_var('subject_checked', '', true));
			$this->bypass = request_var('subject_check_bypass', $this->config['bypass']);

			// Do we even need to be here?
			if (!$this->config['enable'] ||
				(isset($post_data['forum_subject_check']) && empty($post_data['forum_subject_check'])) ||
				$mode != 'post' ||
				empty($subject) ||
				($this->bypass && $subject === $checked) ||
				($this->config['admin'] && $auth->acl_get('a_')) ||
				($this->config['mod'] && $auth->acl_get('m_')) ||
				(!empty($this->config['postcount']) && $user->data['user_posts'] > $this->config['postcount'])
			)
			{
				$this->bypass = true;
				return;
			}

			$this->bypass = true; // This will be set to false later if the situation calls for it.

			$topics = array();
			$sql_select	= ' t.topic_id, t.topic_title, t.forum_id' . ($this->config['fnames'] ? ', f.forum_name' : '');
			$sql_tables	= TOPICS_TABLE . ' t' . ($this->config['fnames'] ? ', ' . FORUMS_TABLE . ' f' : '');
			$sql_forum	= ($this->config['forums'] ? '' : ' AND t.forum_id = ' . $forum_id) . ($this->config['fnames'] ? ' AND f.forum_id = t.forum_id' : '');
			$sql_locked	= $this->config['locked'] ? '' : ' AND t.topic_status <> ' . ITEM_LOCKED;
			$sql_active	= $this->config['active'] ? ' AND t.topic_last_post_time > ' . (time() - ($this->config['active'] * 86400)) : ''; // 86400 = 60 * 60 * 24
			$sql_shadow	= ' AND t.topic_status <> ' . ITEM_MOVED;
			$sql_limit	= $this->config['limit'] ? ' LIMIT ' . $this->config['limit'] : '';
			$sql_order	= ' ORDER BY t.topic_id DESC';
			if ($this->config['strict'])
			{
				// Check for exact subject matches
				$sql = 'SELECT' . $sql_select
				     . '	FROM ' . $sql_tables
				     . '	WHERE t.topic_title = \'' . $db->sql_escape($subject) . '\''
				     . 			$sql_forum
				     . 			$sql_locked
				     .			$sql_shadow
				     . 			$sql_active
				     .		$sql_order
				     . 		$sql_limit
				;
				$result = $db->sql_query($sql);
				while (($row = $db->sql_fetchrow($result)))
				{
					if ($this->config['forums'] && !$auth->acl_get('f_read', $row['forum_id'])) // Skip topics in forums that the user cannot see
					{
						continue;
					}
					$topics[$row['topic_id']] = array(	// Sorted by topic_id
						'topic_id'		=> $row['topic_id'],
						'topic_title' 	=> (isset($row['forum_name']) ? $row['forum_name'] . ': ' : '') . $row['topic_title'],
						'match_count'	=> 0,
					);
				}
				$db->sql_freeresult();
			}
			else
			{
				// Get searchable words from the post subject
				$ignore_words_array = array();
				$synonym_array = array();
				if (file_exists("{$user->lang_path}{$user->lang_name}/search_ignore_words.$phpEx"))
				{
					include("{$user->lang_path}{$user->lang_name}/search_ignore_words.$phpEx");
					$ignore_words_array = &$words;
				}
				if (file_exists("{$user->lang_path}{$user->lang_name}/search_synonyms.$phpEx"))
				{
					include("{$user->lang_path}{$user->lang_name}/search_synonyms.$phpEx");
					$synonym_array = &$synonyms;
				}
				$search_words = $this->clean_and_split_words($subject, $ignore_words_array, $synonym_array);
				$search_words_count = count($search_words);
				unset($ignore_words_array, $synonym_array);

				// Now search for existing topics with subjects containing those words
				$sql = 'SELECT DISTINCT' . $sql_select
					 . '	FROM ' . SEARCH_WORDLIST_TABLE . ' w, ' . SEARCH_WORDMATCH_TABLE . ' m, ' . $sql_tables
					 . '	WHERE (' . $db->sql_in_set('w.word_text', $search_words)
					 . '		AND m.word_id = w.word_id'
					 . '		AND m.title_match = 1'
					 . '		AND m.post_id = t.topic_first_post_id'
					 . '		AND w.word_common <> 1'
					 .			$sql_forum
					 .			$sql_locked
					 .			$sql_shadow
					 .			$sql_active
					 . '	)'
				//	 . '	OR (' . ($count == 0 ? 'topic_title = \'' . $db->sql_escape($subject) . '\'' : '') . $sql_forum . ')' // This could take a real long time on large boards
				     .		$sql_order
					 .		(PRIME_SUBJECT_CHECK_SORT_BEST ? '' : $sql_limit)
				;
				$result = $db->sql_query($sql);
				while (($row = $db->sql_fetchrow($result)))
				{
					if ($this->config['forums'] && !$auth->acl_get('f_read', $row['forum_id'])) // Skip topics in forums that the user cannot see
					{
						continue;
					}
					$title_words = $this->clean_and_split_words($row['topic_title'], $ignore_words_array, $synonym_array);
					$matched_words = array_intersect($search_words, $title_words);
					$matched_count = count($matched_words);
					if (PRIME_SUBJECT_CHECK_MATCH_ALL && $matched_count < $search_words_count)
					{
						continue;
					}
					$topics[$row['topic_id']] = array(
						'topic_id'		=> $row['topic_id'],
						'topic_title' 	=> (isset($row['forum_name']) ? $row['forum_name'] . ': ' : '') . $row['topic_title'],
						'match_count'	=> $matched_count,
					);
				}
				$db->sql_freeresult();
			}
			if (!empty($topics))
			{
				if (PRIME_SUBJECT_CHECK_SORT_BEST)
				{
					// Sort by number of matched words, then by topic_id
					$topic_ids = array();
					$matches = array();
					foreach ($topics as $key => $row)
					{
						$topic_ids[$key] = $row['topic_id'];
						$matches[$key] = $row['match_count'];
					}
					array_multisort($matches, SORT_DESC, $topic_ids, SORT_DESC, $topics);
				}

				// Prepare topic list
				$user->add_lang('mods/prime_subject_check');
				$message = 'SUBJECT_CHECK_SIMILAR_TOPIC';
				$topic_list = '';
				$count = 0;
				foreach ($topics as $row)
				{
					$topic_id		= $row['topic_id'];
					$topic_title	= $row['topic_title'];
					// Check if we need to prevent the post from being made
					if (!$this->config['bypass'] || ($topic_title == $subject && $this->config['bypass'] == PRIME_SUBJECT_CHECK_EXACT))
					{
						$message = 'SUBJECT_CHECK_STRICT_TOPIC';
						$this->bypass = false;
					}
					$count += 1;
					$topic_list .= '<a href="' . append_sid('viewtopic.' . $phpEx . '?t=' . $topic_id) . '" ' . ($this->config['pop'] ? ' onclick="this.target=\'_blank\';"' : '') . '>&bull; ' . $topic_title . '</a><br />';
					if (!empty($this->config['limit']) && $count >= $this->config['limit'])
					{
						break;
					}
				}
				$error[] = $user->lang[$message . ($count == 1 ? '' : 'S')] . ($this->config['pop'] ? $user->lang['SUBJECT_CHECK_NEW_WINDOW'] : $user->lang['SUBJECT_CHECK_SAME_WINDOW']) . '<br />' . $topic_list;
			}
		}

		/**
		* Return a hidden form input which allows the subject check to be bypassed.
		*/
		function add_bypass()
		{
			global $s_hidden_fields; // Global to posting.php

			$s_hidden_fields .= '<input type="hidden" name="subject_checked" value="' . $this->subject . '" />';
			$s_hidden_fields .= '<input type="hidden" name="subject_check_bypass" value="' . ($this->bypass ? 1 : 0) . '" />';

		}

		/**
		* Setup for AJAX.
		*/
		function setup_ajax()
		{
			global $template, $user, $phpbb_root_path, $phpEx;
			global $mode, $post_data, $forum_id; // Global to posting.php

			if ($this->config['ajax'] && $mode == 'post' && (!isset($post_data['forum_subject_check']) || !empty($post_data['forum_subject_check'])))
			{
				$user->add_lang('mods/prime_subject_check');
				$template->assign_var('S_PRIME_SUBJECT_CHECK_AJAX', append_sid("{$phpbb_root_path}prime_subject_check.$phpEx", "f=$forum_id", false, $user->session_id));
			}
		}
	}
	// End class

	$prime_subject_check = new prime_subject_check();
}
?>